@@ -2,7 +2,7 @@
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
 from __future__ import (unicode_literals, division, absolute_import,
                         print_function)
-# from future_builtins import map
+# from polyglot.builtins import map
 
 __license__   = 'GPL v3'
 __copyright__ = '2011, Kovid Goyal <kovid@kovidgoyal.net>'
@@ -21,24 +21,28 @@ from calibre.utils.icu import sort_key
 from calibre.utils.date import UNDEFINED_DATE, clean_date_for_sort, parse_date
 from calibre.utils.localization import calibre_langcode_to_name
 
+
 def bool_sort_key(bools_are_tristate):
     return (lambda x:{True: 1, False: 2, None: 3}.get(x, 3)) if bools_are_tristate else lambda x:{True: 1, False: 2, None: 2}.get(x, 2)
 
+
 IDENTITY = lambda x: x
 
+
 class InvalidLinkTable(Exception):
 
     def __init__(self, name):
         Exception.__init__(self, name)
         self.field_name = name
 
+
 class Field(object):
 
     is_many = False
     is_many_many = False
     is_composite = False
 
-    def __init__(self, name, table, bools_are_tristate):
+    def __init__(self, name, table, bools_are_tristate, get_template_functions):
         self.name, self.table = name, table
         dt = self.metadata['datatype']
         self.has_text_data = dt in {'text', 'comments', 'series', 'enumeration'}
@@ -85,6 +89,7 @@ class Field(object):
             self.category_formatter = calibre_langcode_to_name
         self.writer = Writer(self)
         self.series_field = None
+        self.get_template_functions = get_template_functions
 
     @property
     def metadata(self):
@@ -166,6 +171,7 @@ class Field(object):
                 ans.append(c)
         return ans
 
+
 class OneToOneField(Field):
 
     def for_book(self, book_id, default_value=None):
@@ -193,13 +199,14 @@ class OneToOneField(Field):
         for book_id in candidates:
             yield cbm.get(book_id, default_value), {book_id}
 
+
 class CompositeField(OneToOneField):
 
     is_composite = True
     SIZE_SUFFIX_MAP = {suffix:i for i, suffix in enumerate(('', 'K', 'M', 'G', 'T', 'P', 'E'))}
 
-    def __init__(self, name, table, bools_are_tristate):
-        OneToOneField.__init__(self, name, table, bools_are_tristate)
+    def __init__(self, name, table, bools_are_tristate, get_template_functions):
+        OneToOneField.__init__(self, name, table, bools_are_tristate, get_template_functions)
 
         self._render_cache = {}
         self._lock = Lock()
@@ -259,7 +266,8 @@ class CompositeField(OneToOneField):
         ' INTERNAL USE ONLY. DO NOT USE THIS OUTSIDE THIS CLASS! '
         ans = formatter.safe_format(
             self.metadata['display']['composite_template'], mi, _('TEMPLATE ERROR'),
-            mi, column_name=self._composite_name, template_cache=template_cache).strip()
+            mi, column_name=self._composite_name, template_cache=template_cache,
+            template_functions=self.get_template_functions()).strip()
         with self._lock:
             self._render_cache[book_id] = ans
         return ans
@@ -339,9 +347,10 @@ class CompositeField(OneToOneField):
                 ans.add(book_id)
         return ans
 
+
 class OnDeviceField(OneToOneField):
 
-    def __init__(self, name, table, bools_are_tristate):
+    def __init__(self, name, table, bools_are_tristate, get_template_functions):
         self.name = name
         self.book_on_device_func = None
         self.is_multiple = False
@@ -403,6 +412,7 @@ class OnDeviceField(OneToOneField):
         for val, book_ids in val_map.iteritems():
             yield val, book_ids
 
+
 class LazySortMap(object):
 
     __slots__ = ('default_sort_key', 'sort_key_func', 'id_map', 'cache')
@@ -469,6 +479,7 @@ class ManyToOneField(Field):
         except KeyError:
             raise InvalidLinkTable(self.name)
 
+
 class ManyToManyField(Field):
 
     is_many = True
@@ -534,6 +545,7 @@ class ManyToManyField(Field):
         except KeyError:
             raise InvalidLinkTable(self.name)
 
+
 class IdentifiersField(ManyToManyField):
 
     def for_book(self, book_id, default_value=None):
@@ -569,6 +581,7 @@ class IdentifiersField(ManyToManyField):
                 ans.append(c)
         return ans
 
+
 class AuthorsField(ManyToManyField):
 
     def author_data(self, author_id):
@@ -588,6 +601,7 @@ class AuthorsField(ManyToManyField):
         return ' & '.join(self.table.asort_map[k] for k in
                           self.table.book_col_map[book_id])
 
+
 class FormatsField(ManyToManyField):
 
     def for_book(self, book_id, default_value=None):
@@ -618,6 +632,7 @@ class FormatsField(ManyToManyField):
                 ans.append(c)
         return ans
 
+
 class LazySeriesSortMap(object):
 
     __slots__ = ('default_sort_key', 'sort_key_func', 'id_map', 'cache')
@@ -638,12 +653,14 @@ class LazySeriesSortMap(object):
                 val = self.cache[(item_id, lang)] = self.default_sort_key
             return val
 
+
 class SeriesField(ManyToOneField):
 
     def sort_keys_for_books(self, get_metadata, lang_map):
         sso = tweaks['title_series_sorting']
         ssk = self._sort_key
         ts = title_sort
+
         def sk(val, lang):
             return ssk(ts(val, order=sso, lang=lang))
         sk_map = LazySeriesSortMap(self._default_sort_key, sk, self.table.id_map)
@@ -718,7 +735,8 @@ class TagsField(ManyToManyField):
                 ans.append(c)
         return ans
 
-def create_field(name, table, bools_are_tristate):
+
+def create_field(name, table, bools_are_tristate, get_template_functions):
     cls = {
             ONE_ONE: OneToOneField,
             MANY_ONE: ManyToOneField,
@@ -738,5 +756,4 @@ def create_field(name, table, bools_are_tristate):
         cls = CompositeField
     elif table.metadata['datatype'] == 'series':
         cls = SeriesField
-    return cls(name, table, bools_are_tristate)
-
+    return cls(name, table, bools_are_tristate, get_template_functions)
