@@ -1,6 +1,7 @@
 #!/usr/bin/env python2
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
 from __future__ import with_statement
+from __future__ import print_function
 
 __license__   = 'GPL v3'
 __copyright__ = '2009, Kovid Goyal <kovid@kovidgoyal.net>'
@@ -37,13 +38,14 @@ class PluginWidget(QWidget,Ui_Form):
     sync_enabled = True
 
     # Formats supported by this plugin
-    formats = set(['azw3','epub','mobi'])
+    formats = {'azw3','epub','mobi'}
 
     def __init__(self, parent=None):
         QWidget.__init__(self, parent)
         self.setupUi(self)
         self._initControlArrays()
         self.blocking_all_signals = None
+        self.parent_ref = lambda: None
 
     def _initControlArrays(self):
         # Default values for controls
@@ -139,18 +141,17 @@ class PluginWidget(QWidget,Ui_Form):
             # Skip disabled and incomplete rules
             if not rule['enabled']:
                 continue
-            elif not rule['field'] or not rule['pattern']:
+            if not rule['field'] or not rule['pattern']:
                 continue
-            elif 'prefix' in rule and rule['prefix'] is None:
+            if 'prefix' in rule and rule['prefix'] is None:
                 continue
-            else:
-                if rule['field'] != _('Tags'):
-                    # Look up custom column friendly name
-                    rule['field'] = self.eligible_custom_fields[rule['field']]['field']
-                    if rule['pattern'] in [_('any value'),_('any date')]:
-                        rule['pattern'] = '.*'
-                    elif rule['pattern'] == _('unspecified'):
-                        rule['pattern'] = 'None'
+            if rule['field'] != _('Tags'):
+                # Look up custom column friendly name
+                rule['field'] = self.eligible_custom_fields[rule['field']]['field']
+                if rule['pattern'] in [_('any value'),_('any date')]:
+                    rule['pattern'] = '.*'
+                elif rule['pattern'] == _('unspecified'):
+                    rule['pattern'] = 'None'
             if 'prefix' in rule:
                 pr = (rule['name'],rule['field'],rule['pattern'],rule['prefix'])
             else:
@@ -283,17 +284,10 @@ class PluginWidget(QWidget,Ui_Form):
     def get_format_and_title(self):
         current_format = None
         current_title = None
-        self.parentWidget().blockSignals(True)
-        for peer in self.parentWidget().children():
-            if peer == self:
-                continue
-            elif peer.children():
-                for child in peer.children():
-                    if child.objectName() == 'format':
-                        current_format = child.currentText().strip()
-                    elif child.objectName() == 'title':
-                        current_title = unicode(child.text()).strip()
-        self.parentWidget().blockSignals(False)
+        parent = self.parent_ref()
+        if parent is not None:
+            current_title = parent.title.text().strip()
+            current_format = parent.format.currentText().strip()
         return current_format, current_title
 
     def header_note_source_field_changed(self,new_index):
@@ -521,9 +515,9 @@ class PluginWidget(QWidget,Ui_Form):
             opts_dict['output_profile'] = ['default']
 
         if False and self.DEBUG:
-            print "opts_dict"
+            print("opts_dict")
             for opt in sorted(opts_dict.keys(), key=sort_key):
-                print " %s: %s" % (opt, repr(opts_dict[opt]))
+                print(" %s: %s" % (opt, repr(opts_dict[opt])))
         return opts_dict
 
     def populate_combo_boxes(self):
@@ -795,18 +789,15 @@ class PluginWidget(QWidget,Ui_Form):
         self.preset_field.setCurrentIndex(self.preset_field.findText(name))
 
     def set_format_and_title(self, format, title):
-        for peer in self.parentWidget().children():
-            if peer == self:
-                continue
-            elif peer.children():
-                for child in peer.children():
-                    if child.objectName() == 'format':
-                        index = child.findText(format)
-                        child.blockSignals(True)
-                        child.setCurrentIndex(index)
-                        child.blockSignals(False)
-                    elif child.objectName() == 'title':
-                        child.setText(title)
+        parent = self.parent_ref()
+        if parent is not None:
+            if format:
+                index = parent.format.findText(format)
+                parent.format.blockSignals(True)
+                parent.format.setCurrentIndex(index)
+                parent.format.blockSignals(False)
+            if title:
+                parent.title.setText(title)
 
     def settings_changed(self, source):
         '''
