@@ -18,7 +18,8 @@ from calibre.constants import iswindows
 from calibre.gui2.widgets import EnLineEdit
 from calibre.gui2.widgets2 import populate_standard_spinbox_context_menu, RatingEditor
 from calibre.gui2.complete2 import EditWithComplete
-from calibre.utils.date import now, format_date, qt_to_dt, is_date_undefined
+from calibre.utils.date import now, format_date, qt_to_dt, is_date_undefined, internal_iso_format_string
+
 from calibre.utils.config import tweaks
 from calibre.utils.icu import sort_key
 from calibre.gui2.dialogs.comments_dialog import CommentsDialog, PlainTextDialog
@@ -64,8 +65,18 @@ class UpdateEditorGeometry(object):
             new_width += r.width()
 
         # Compute the maximum we can show if we consume the entire viewport
-        max_width = (self.table_widget.horizontalScrollBar().geometry().width() -
-                     self.table_widget.verticalHeader().width())
+        pin_view = self.table_widget.pin_view
+        is_pin_view, p = False, editor.parent()
+        while p is not None:
+            if p is pin_view:
+                is_pin_view = True
+                break
+            p = p.parent()
+        if is_pin_view:
+            max_width = pin_view.horizontalScrollBar().geometry().width()
+        else:
+            view = self.table_widget
+            max_width = view.horizontalScrollBar().geometry().width() - view.verticalHeader().width()
         # What we have to display might not fit. If so, adjust down
         new_width = new_width if new_width < max_width else max_width
 
@@ -101,22 +112,29 @@ class UpdateEditorGeometry(object):
 
 class DateTimeEdit(QDateTimeEdit):  # {{{
 
-    def __init__(self, parent, format):
+    def __init__(self, parent, format_):
         QDateTimeEdit.__init__(self, parent)
         self.setFrame(False)
         self.setMinimumDateTime(UNDEFINED_QDATETIME)
         self.setSpecialValueText(_('Undefined'))
         self.setCalendarPopup(True)
-        self.setDisplayFormat(format)
+        if format_ == 'iso':
+            format_ = internal_iso_format_string()
+        self.setDisplayFormat(format_)
 
     def contextMenuEvent(self, ev):
         m = QMenu(self)
         m.addAction(_('Set date to undefined') + '\t' + QKeySequence(Qt.Key_Minus).toString(QKeySequence.NativeText),
                     self.clear_date)
+        m.addAction(_('Set date to today') + '\t' + QKeySequence(Qt.Key_Equal).toString(QKeySequence.NativeText),
+                    self.today_date)
         m.addSeparator()
         populate_standard_spinbox_context_menu(self, m)
         m.popup(ev.globalPos())
 
+    def today_date(self):
+        self.setDateTime(QDateTime.currentDateTime())
+
     def clear_date(self):
         self.setDateTime(UNDEFINED_QDATETIME)
 
@@ -125,8 +143,8 @@ class DateTimeEdit(QDateTimeEdit):  # {{{
             ev.accept()
             self.clear_date()
         elif ev.key() == Qt.Key_Equal:
+            self.today_date()
             ev.accept()
-            self.setDateTime(QDateTime.currentDateTime())
         else:
             return QDateTimeEdit.keyPressEvent(self, ev)
 # }}}
@@ -156,6 +174,7 @@ def make_clearing_spinbox(spinbox):
                 return spinbox.keyPressEvent(self, ev)
     return SpinBox
 
+
 ClearingSpinBox = make_clearing_spinbox(QSpinBox)
 ClearingDoubleSpinBox = make_clearing_spinbox(QDoubleSpinBox)
 
@@ -282,7 +301,7 @@ class PubDateDelegate(QStyledItemDelegate, UpdateEditorGeometry):  # {{{
         elif check_key_modifier(Qt.ShiftModifier + Qt.ControlModifier):
             val = now()
         elif is_date_undefined(val):
-            val = QDate(2000, 1, 1)
+            val = QDate.currentDate()
         if isinstance(val, QDateTime):
             val = val.date()
         editor.setDate(val)
@@ -413,11 +432,13 @@ class CcDateDelegate(QStyledItemDelegate, UpdateEditorGeometry):  # {{{
         QStyledItemDelegate.__init__(self, parent)
         self.table_widget = parent
 
-    def set_format(self, format):
-        if not format:
+    def set_format(self, _format):
+        if not _format:
             self.format = 'dd MMM yyyy'
+        elif _format == 'iso':
+            self.format = internal_iso_format_string()
         else:
-            self.format = format
+            self.format = _format
 
     def displayText(self, val, locale):
         d = qt_to_dt(val)
