@@ -69,7 +69,7 @@ class TagBrowserMixin(object):  # {{{
                     m.addAction(name, partial(self.do_tags_list_edit, None, cat))
 
     def init_tag_browser_mixin(self, db):
-        self.library_view.model().count_changed_signal.connect(self.tags_view.recount)
+        self.library_view.model().count_changed_signal.connect(self.tags_view.recount_with_position_based_index)
         self.tags_view.set_database(db, self.alter_tb)
         self.tags_view.tags_marked.connect(self.search.set_search_string)
         self.tags_view.tags_list_edit.connect(self.do_tags_list_edit)
@@ -135,7 +135,7 @@ class TagBrowserMixin(object):  # {{{
 
     def do_edit_user_categories(self, on_category=None):
         '''
-        Open the user categories editor.
+        Open the User categories editor.
         '''
         db = self.library_view.model().db
         d = TagCategories(self, db, on_category,
@@ -151,7 +151,7 @@ class TagBrowserMixin(object):  # {{{
 
     def do_delete_user_category(self, category_name):
         '''
-        Delete the user category named category_name. Any leading '@' is removed
+        Delete the User category named category_name. Any leading '@' is removed
         '''
         if category_name.startswith('@'):
             category_name = category_name[1:]
@@ -167,10 +167,10 @@ class TagBrowserMixin(object):  # {{{
             elif k.startswith(category_name + '.'):
                 has_children = True
         if not found:
-            return error_dialog(self.tags_view, _('Delete user category'),
-                         _('%s is not a user category')%category_name, show=True)
+            return error_dialog(self.tags_view, _('Delete User category'),
+                         _('%s is not a User category')%category_name, show=True)
         if has_children:
-            if not question_dialog(self.tags_view, _('Delete user category'),
+            if not question_dialog(self.tags_view, _('Delete User category'),
                                    _('%s contains items. Do you really '
                                      'want to delete it?')%category_name):
                 return
@@ -185,7 +185,7 @@ class TagBrowserMixin(object):  # {{{
 
     def do_del_item_from_user_cat(self, user_cat, item_name, item_category):
         '''
-        Delete the item (item_name, item_category) from the user category with
+        Delete the item (item_name, item_category) from the User category with
         key user_cat. Any leading '@' characters are removed
         '''
         if user_cat.startswith('@'):
@@ -204,7 +204,7 @@ class TagBrowserMixin(object):  # {{{
 
     def do_add_item_to_user_cat(self, dest_category, src_name, src_category):
         '''
-        Add the item src_name in src_category to the user category
+        Add the item src_name in src_category to the User category
         dest_category. Any leading '@' is removed
         '''
         db = self.library_view.model().db
@@ -214,10 +214,10 @@ class TagBrowserMixin(object):  # {{{
             dest_category = dest_category[1:]
 
         if dest_category not in user_cats:
-            return error_dialog(self.tags_view, _('Add to user category'),
-                    _('A user category %s does not exist')%dest_category, show=True)
+            return error_dialog(self.tags_view, _('Add to User category'),
+                    _('A User category %s does not exist')%dest_category, show=True)
 
-        # Now add the item to the destination user category
+        # Now add the item to the destination User category
         add_it = True
         if src_category == 'news':
             src_category = 'tags'
@@ -237,13 +237,15 @@ class TagBrowserMixin(object):  # {{{
         '''
 
         db = self.current_db
-        data = db.new_api.get_categories()
-        if category in data:
-            result = [(t.id, t.original_name, t.count) for t in data[category] if t.count > 0]
-        else:
-            result = None
-        if result is None:
-            return
+
+        def get_book_ids(use_virtual_library):
+            book_ids = None if not use_virtual_library else self.tags_view.model().get_book_ids_to_use()
+            data = db.new_api.get_categories(book_ids=book_ids)
+            if category in data:
+                result = [(t.id, t.original_name, t.count) for t in data[category] if t.count > 0]
+            else:
+                result = None
+            return result
 
         if category == 'series':
             key = lambda x:sort_key(title_sort(x))
@@ -251,7 +253,7 @@ class TagBrowserMixin(object):  # {{{
             key = sort_key
 
         d = TagListEditor(self, cat_name=db.field_metadata[category]['name'],
-                          tag_to_match=tag, data=result, sorter=key)
+                          tag_to_match=tag, get_book_ids=get_book_ids, sorter=key)
         d.exec_()
         if d.result() == d.Accepted:
             to_rename = d.to_rename  # dict of old id to new name
@@ -399,6 +401,7 @@ class TagBrowserBar(QWidget):  # {{{
         ac.triggered.connect(self.set_focus_to_find_box)
 
         self.search_button = QToolButton()
+        self.search_button.setAutoRaise(True)
         self.search_button.setCursor(Qt.PointingHandCursor)
         self.search_button.setIcon(QIcon(I('search.png')))
         self.search_button.setToolTip(_('Find the first/next matching item'))
@@ -435,8 +438,6 @@ class TagBrowserBar(QWidget):  # {{{
     def update_searchbar_state(self):
         find_shown = self.toggle_search_button.isChecked()
         self.toggle_search_button.setVisible(not find_shown)
-        self.search_button.setVisible(find_shown)
-        self.item_search.setVisible(find_shown)
         l = self.layout()
         items = [l.itemAt(i) for i in range(l.count())]
         tuple(map(l.removeItem, items))
@@ -446,12 +447,19 @@ class TagBrowserBar(QWidget):  # {{{
             l.addWidget(self.item_search, 10)
             l.addWidget(self.search_button)
             self.item_search.setFocus(Qt.OtherFocusReason)
+            self.toggle_search_button.setVisible(False)
+            self.search_button.setVisible(True)
+            self.item_search.setVisible(True)
         else:
             l.addWidget(self.alter_tb)
             self.alter_tb.setToolButtonStyle(Qt.ToolButtonTextBesideIcon)
             l.addStretch(10)
             l.addStretch(10)
             l.addWidget(self.toggle_search_button)
+            self.toggle_search_button.setVisible(True)
+            self.search_button.setVisible(False)
+            self.item_search.setVisible(False)
+
 # }}}
 
 
@@ -472,7 +480,6 @@ class TagBrowserWidget(QFrame):  # {{{
 
         self.current_find_position = None
         self.search_button.clicked.connect(self.find)
-        self.item_search.lineEdit().returnPressed.connect(self.do_find)
         self.item_search.lineEdit().textEdited.connect(self.find_text_changed)
         self.item_search.activated[str].connect(self.do_find)
 
@@ -503,7 +510,7 @@ class TagBrowserWidget(QFrame):  # {{{
         parent.keyboard.register_shortcut('tag browser collapse all',
                 _('Collapse all'), default_keys=(),
                 action=ac, group=_('Tag browser'))
-        ac.triggered.connect(lambda : self.tags_view.collapseAll())
+        connect_lambda(ac.triggered, self, lambda self: self.tags_view.collapseAll())
 
         ac = QAction(parent)
         parent.addAction(ac)
@@ -582,10 +589,14 @@ class TagBrowserWidget(QFrame):  # {{{
         self.current_find_position = None
         self.find()
 
+    @property
+    def find_text(self):
+        return unicode(self.item_search.currentText()).strip()
+
     def find(self):
         model = self.tags_view.model()
         model.clear_boxed()
-        txt = unicode(self.item_search.currentText()).strip()
+        txt = self.find_text
 
         if txt.startswith('*'):
             model.set_categories_filter(txt[1:])
@@ -631,4 +642,12 @@ class TagBrowserWidget(QFrame):  # {{{
     def not_found_label_timer_event(self):
         self.not_found_label.setVisible(False)
 
+    def keyPressEvent(self, ev):
+        if ev.key() in (Qt.Key_Enter, Qt.Key_Return) and self.find_text:
+            self.find()
+            ev.accept()
+            return
+        return QFrame.keyPressEvent(self, ev)
+
+
 # }}}
