@@ -396,7 +396,7 @@ class BasicNewsRecipe(Recipe):
     # See the built-in recipes for examples of these settings.
 
     def short_title(self):
-        return self.title
+        return force_unicode(self.title, preferred_encoding)
 
     def is_link_wanted(self, url, tag):
         '''
@@ -407,7 +407,7 @@ class BasicNewsRecipe(Recipe):
         :param url: The URL to be followed
         :param tag: The tag from which the URL was derived
         '''
-        raise NotImplementedError
+        raise NotImplementedError()
 
     def get_extra_css(self):
         '''
@@ -452,7 +452,7 @@ class BasicNewsRecipe(Recipe):
         return self.feeds
 
     @classmethod
-    def print_version(self, url):
+    def print_version(cls, url):
         '''
         Take a `url` pointing to the webpage with article content and return the
         :term:`URL` pointing to the print version of the article. By default does
@@ -483,7 +483,7 @@ class BasicNewsRecipe(Recipe):
     def get_browser(self, *args, **kwargs):
         '''
         Return a browser instance used to fetch documents from the web. By default
-        it returns a `mechanize <https://github.com/jjlee/mechanize>`_
+        it returns a `mechanize <https://mechanize.readthedocs.io/en/latest/>`_
         browser instance that supports cookies, ignores robots.txt, handles
         refreshes and has a mozilla firefox user agent.
 
@@ -657,7 +657,7 @@ class BasicNewsRecipe(Recipe):
             return frozenset()
         return frozenset([(parts.netloc, (parts.path or '').rstrip('/'))])
 
-    def index_to_soup(self, url_or_raw, raw=False, as_tree=False):
+    def index_to_soup(self, url_or_raw, raw=False, as_tree=False, save_raw=None):
         '''
         Convenience method that takes an URL to the index page and returns
         a `BeautifulSoup <https://www.crummy.com/software/BeautifulSoup/bs3/documentation.html>`_
@@ -685,21 +685,23 @@ class BasicNewsRecipe(Recipe):
                 _raw = self.encoding(_raw)
             else:
                 _raw = _raw.decode(self.encoding, 'replace')
+        from calibre.ebooks.chardet import strip_encoding_declarations, xml_to_unicode
+        from calibre.utils.cleantext import clean_xml_chars
+        if isinstance(_raw, unicode):
+            _raw = strip_encoding_declarations(_raw)
+        else:
+            _raw = xml_to_unicode(_raw, strip_encoding_pats=True, resolve_entities=True)[0]
+        _raw = clean_xml_chars(_raw)
+        if save_raw:
+            with lopen(save_raw, 'wb') as f:
+                f.write(_raw.encode('utf-8'))
         if as_tree:
-            import html5lib
-            from calibre.ebooks.chardet import strip_encoding_declarations, xml_to_unicode
-            from calibre.utils.cleantext import clean_xml_chars
-            if isinstance(_raw, unicode):
-                _raw = strip_encoding_declarations(_raw)
-            else:
-                _raw = xml_to_unicode(_raw, strip_encoding_pats=True, resolve_entities=True)[0]
-            return html5lib.parse(clean_xml_chars(_raw), treebuilder='lxml', namespaceHTMLElements=False)
-
-        massage = list(BeautifulSoup.MARKUP_MASSAGE)
-        enc = 'cp1252' if callable(self.encoding) or self.encoding is None else self.encoding
-        massage.append((re.compile(r'&(\S+?);'), lambda match:
-            entity_to_unicode(match, encoding=enc)))
-        return BeautifulSoup(_raw, markupMassage=massage)
+            from html5_parser import parse
+            return parse(_raw)
+        else:
+            from html5_parser.soup import set_soup_module, parse
+            set_soup_module(sys.modules[BeautifulSoup.__module__])
+            return parse(_raw, return_root=False)
 
     def extract_readable_article(self, html, url):
         '''
@@ -861,7 +863,7 @@ class BasicNewsRecipe(Recipe):
         '''
         Initialize the recipe.
         :param options: Parsed commandline options
-        :param parser:  Command line option parser. Used to intelligently merge options.
+        :param log:  Logging object
         :param progress_reporter: A Callable that takes two arguments: progress (a number between 0 and 1) and a string message. The message should be optional.
         '''
         self.log = ThreadSafeWrapper(log)
@@ -890,14 +892,9 @@ class BasicNewsRecipe(Recipe):
             self.verbose = True
         self.report_progress = progress_reporter
 
-        if isinstance(self.feeds, basestring):
-            self.feeds = eval(self.feeds)
-            if isinstance(self.feeds, basestring):
-                self.feeds = [self.feeds]
-
         if self.needs_subscription and (
-                self.username is None or self.password is None or
-                (not self.username and not self.password)):
+                self.username is None or self.password is None or (
+                    not self.username and not self.password)):
             if self.needs_subscription != 'optional':
                 raise ValueError(_('The "%s" recipe needs a username and password.')%self.title)
 
@@ -989,6 +986,11 @@ class BasicNewsRecipe(Recipe):
         for base in list(soup.findAll(['base', 'iframe', 'canvas', 'embed',
             'command', 'datalist', 'video', 'audio'])):
             base.extract()
+        # srcset causes some viewers, like calibre's to load images from the
+        # web, and it also possible causes iBooks on iOS to barf, see
+        # https://bugs.launchpad.net/bugs/1713986
+        for img in soup.findAll('img', srcset=True):
+            del img['srcset']
 
         ans = self.postprocess_html(soup, first_fetch)
 
@@ -1430,11 +1432,11 @@ class BasicNewsRecipe(Recipe):
                     aseen.add(a.title)
                     article_titles.append(force_unicode(a.title, 'utf-8'))
 
-        mi.comments = self.description
-        if not isinstance(mi.comments, unicode):
-            mi.comments = mi.comments.decode('utf-8', 'replace')
-        mi.comments += ('\n\n' + _('Articles in this issue: ') + '\n' +
-                '\n\n'.join(article_titles))
+        desc = self.description
+        if not isinstance(desc, unicode):
+            desc = desc.decode('utf-8', 'replace')
+        mi.comments = (_('Articles in this issue:'
+            ) + '\n\n' + '\n\n'.join(article_titles)) + '\n\n' + desc
 
         language = canonicalize_lang(self.language)
         if language is not None:
@@ -1634,8 +1636,7 @@ class BasicNewsRecipe(Recipe):
                 parsed_feeds.append(feed)
                 self.log.exception(msg)
 
-        remove = [fl for fl in parsed_feeds if len(fl) == 0 and
-                self.remove_empty_feeds]
+        remove = [fl for fl in parsed_feeds if len(fl) == 0 and self.remove_empty_feeds]
         for f in remove:
             parsed_feeds.remove(f)
 
@@ -1683,7 +1684,7 @@ class BasicNewsRecipe(Recipe):
 
     @classmethod
     def soup(cls, raw):
-        entity_replace = [(re.compile(ur'&(\S+?);'), partial(entity_to_unicode,
+        entity_replace = [(re.compile(u'&(\\S+?);'), partial(entity_to_unicode,
                                                            exceptions=[]))]
         nmassage = list(BeautifulSoup.MARKUP_MASSAGE)
         nmassage.extend(entity_replace)
